# Visualizing statistical relationships

Statistical analysis is a process of understanding how variables in a dataset relate to each other and how those relationships depend on other variables. Visualization can be a core component of this process because, when data are visualized properly, the human visual system can see trends and patterns that indicate a relationship.

We will discuss three seaborn functions in this tutorial. The one we will use most is [`relplot()`](../generated/seaborn.relplot.html#seaborn.relplot "seaborn.relplot"). This is a [figure-level function](../introduction.html#intro-func-types) for visualizing statistical relationships using two common approaches: scatter plots and line plots. [`relplot()`](../generated/seaborn.relplot.html#seaborn.relplot "seaborn.relplot") combines a [`FacetGrid`](../generated/seaborn.FacetGrid.html#seaborn.FacetGrid "seaborn.FacetGrid") with one of two axes-level functions:

*   [`scatterplot()`](../generated/seaborn.scatterplot.html#seaborn.scatterplot "seaborn.scatterplot") (with `kind="scatter"`; the default)
*   [`lineplot()`](../generated/seaborn.lineplot.html#seaborn.lineplot "seaborn.lineplot") (with `kind="line"`)

As we will see, these functions can be quite illuminating because they use simple and easily-understood representations of data that can nevertheless represent complex dataset structures. They can do so because they plot two-dimensional graphics that can be enhanced by mapping up to three additional variables using the semantics of hue, size, and style.

```py
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
sns.set(style="darkgrid")

```

## Relating variables with scatter plots

The scatter plot is a mainstay of statistical visualization. It depicts the joint distribution of two variables using a cloud of points, where each point represents an observation in the dataset. This depiction allows the eye to infer a substantial amount of information about whether there is any meaningful relationship between them.

There are several ways to draw a scatter plot in seaborn. The most basic, which should be used when both variables are numeric, is the [`scatterplot()`](../generated/seaborn.scatterplot.html#seaborn.scatterplot "seaborn.scatterplot") function. In the [categorical visualization tutorial](categorical.html#categorical-tutorial), we will see specialized tools for using scatterplots to visualize categorical data. The [`scatterplot()`](../generated/seaborn.scatterplot.html#seaborn.scatterplot "seaborn.scatterplot") is the default `kind` in [`relplot()`](../generated/seaborn.relplot.html#seaborn.relplot "seaborn.relplot") (it can also be forced by setting `kind="scatter"`):

```py
tips = sns.load_dataset("tips")
sns.relplot(x="total_bill", y="tip", data=tips);

```

![http://seaborn.pydata.org/_images/relational_4_0.png](img/da2b66e95b2bbd08f2c58046234e97b1.jpg)

While the points are plotted in two dimensions, another dimension can be added to the plot by coloring the points according to a third variable. In seaborn, this is referred to as using a “hue semantic”, because the color of the point gains meaning:

```py
sns.relplot(x="total_bill", y="tip", hue="smoker", data=tips);

```

![http://seaborn.pydata.org/_images/relational_6_0.png](img/7e2cd86a1f38e486133cd69728feed62.jpg)

To emphasize the difference between the classes, and to improve accessibility, you can use a different marker style for each class:

```py
sns.relplot(x="total_bill", y="tip", hue="smoker", style="smoker",
            data=tips);

```

![http://seaborn.pydata.org/_images/relational_8_0.png](img/9240094a48ef05fa3a34fab7ea6bb093.jpg)

It’s also possible to represent four variables by changing the hue and style of each point independently. But this should be done carefully, because the eye is much less sensitive to shape than to color:

```py
sns.relplot(x="total_bill", y="tip", hue="smoker", style="time", data=tips);

```

![http://seaborn.pydata.org/_images/relational_10_0.png](img/36180ad55b36cb767608c261d92e2918.jpg)

In the examples above, the hue semantic was categorical, so the default [qualitative palette](color_palettes.html#palette-tutorial) was applied. If the hue semantic is numeric (specifically, if it can be cast to float), the default coloring switches to a sequential palette:

```py
sns.relplot(x="total_bill", y="tip", hue="size", data=tips);

```

![http://seaborn.pydata.org/_images/relational_12_0.png](img/e2261011757746831cd8de1e799bf2e5.jpg)

In both cases, you can customize the color palette. There are many options for doing so. Here, we customize a sequential palette using the string interface to [`cubehelix_palette()`](../generated/seaborn.cubehelix_palette.html#seaborn.cubehelix_palette "seaborn.cubehelix_palette"):

```py
sns.relplot(x="total_bill", y="tip", hue="size", palette="ch:r=-.5,l=.75", data=tips);

```

![http://seaborn.pydata.org/_images/relational_14_0.png](img/3a9f5f15fbcf65ae0763da92544620c3.jpg)

The third kind of semantic variable changes the size of each point:

```py
sns.relplot(x="total_bill", y="tip", size="size", data=tips);

```

![http://seaborn.pydata.org/_images/relational_16_0.png](img/c89bfa6f82af190645b6a97983037d2f.jpg)

Unlike with [`matplotlib.pyplot.scatter()`](https://matplotlib.org/api/_as_gen/matplotlib.pyplot.scatter.html#matplotlib.pyplot.scatter "(in Matplotlib v2.2.2)"), the literal value of the variable is not used to pick the area of the point. Instead, the range of values in data units is normalized into a range in area units. This range can be customized:

```py
sns.relplot(x="total_bill", y="tip", size="size", sizes=(15, 200), data=tips);

```

![http://seaborn.pydata.org/_images/relational_18_0.png](img/c622412b599649be9b4d765223de73f5.jpg)

More examples for customizing how the different semantics are used to show statistical relationships are shown in the [`scatterplot()`](../generated/seaborn.scatterplot.html#seaborn.scatterplot "seaborn.scatterplot") API examples.

## Emphasizing continuity with line plots

Scatter plots are highly effective, but there is no universally optimal type of visualiation. Instead, the visual representation should be adapted for the specifics of the dataset and to the question you are trying to answer with the plot.

With some datasets, you may want to understand changes in one variable as a function of time, or a similarly continuous variable. In this situation, a good choice is to draw a line plot. In seaborn, this can be accomplished by the [`lineplot()`](../generated/seaborn.lineplot.html#seaborn.lineplot "seaborn.lineplot") function, either directly or with [`relplot()`](../generated/seaborn.relplot.html#seaborn.relplot "seaborn.relplot") by setting `kind="line"`:

```py
df = pd.DataFrame(dict(time=np.arange(500),
                       value=np.random.randn(500).cumsum()))
g = sns.relplot(x="time", y="value", kind="line", data=df)
g.fig.autofmt_xdate()

```

![http://seaborn.pydata.org/_images/relational_21_0.png](img/41ec1c6b7e5de5ac6cc5f6020ac029fc.jpg)

Because [`lineplot()`](../generated/seaborn.lineplot.html#seaborn.lineplot "seaborn.lineplot") assumes that you are most often trying to draw `y` as a function of `x`, the default behavior is to sort the data by the `x` values before plotting. However, this can be disabled:

```py
df = pd.DataFrame(np.random.randn(500, 2).cumsum(axis=0), columns=["x", "y"])
sns.relplot(x="x", y="y", sort=False, kind="line", data=df);

```

![http://seaborn.pydata.org/_images/relational_23_0.png](img/2d731687030e9e2dab13cbdb46017e91.jpg)

### Aggregation and representing uncertainty

More complex datasets will have multiple measurements for the same value of the `x` variable. The default behavior in seaborn is to aggregate the multiple measurements at each `x` value by plotting the mean and the 95% confidence interval around the mean:

```py
fmri = sns.load_dataset("fmri")
sns.relplot(x="timepoint", y="signal", kind="line", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_25_0.png](img/c1b2a3e1708607a80c705b690a9c6c5a.jpg)

The confidence intervals are computed using bootstrapping, which can be time-intensive for larger datasets. It’s therefore possible to disable them:

```py
sns.relplot(x="timepoint", y="signal", ci=None, kind="line", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_27_0.png](img/95ca0b5950c77a49aceb9f22e148ba9c.jpg)

Another good option, especially with larger data, is to represent the spread of the distribution at each timepoint by plotting the standard deviation instead of a confidence interval:

```py
sns.relplot(x="timepoint", y="signal", kind="line", ci="sd", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_29_0.png](img/318e42d583df0fc8e01c6d7c565d758c.jpg)

To turn off aggregation altogether, set the `estimator` parameter to `None` This might produce a strange effect when the data have multiple observations at each point.

```py
sns.relplot(x="timepoint", y="signal", estimator=None, kind="line", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_31_0.png](img/5edaaa0dfdffc4b78552a1a5b68f5491.jpg)

### Plotting subsets of data with semantic mappings

The [`lineplot()`](../generated/seaborn.lineplot.html#seaborn.lineplot "seaborn.lineplot") function has the same flexibility as [`scatterplot()`](../generated/seaborn.scatterplot.html#seaborn.scatterplot "seaborn.scatterplot"): it can show up to three additional variables by modifying the hue, size, and style of the plot elements. It does so using the same API as [`scatterplot()`](../generated/seaborn.scatterplot.html#seaborn.scatterplot "seaborn.scatterplot"), meaning that we don’t need to stop and think about the parameters that control the look of lines vs. points in matplotlib.

Using semantics in [`lineplot()`](../generated/seaborn.lineplot.html#seaborn.lineplot "seaborn.lineplot") will also determine how the data get aggregated. For example, adding a hue semantic with two levels splits the plot into two lines and error bands, coloring each to indicate which subset of the data they correspond to.

```py
sns.relplot(x="timepoint", y="signal", hue="event", kind="line", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_33_0.png](img/a9e30b276189c5c26d3a9959798a84a6.jpg)

Adding a style semantic to a line plot changes the pattern of dashes in the line by default:

```py
sns.relplot(x="timepoint", y="signal", hue="region", style="event",
            kind="line", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_35_0.png](img/b51b0bae21d924ef1f739aabdd811e31.jpg)

But you can identify subsets by the markers used at each observation, either together with the dashes or instead of them:

```py
sns.relplot(x="timepoint", y="signal", hue="region", style="event",
            dashes=False, markers=True, kind="line", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_37_0.png](img/68010acf9a074a4aa94ad7e89b9a819e.jpg)

As with scatter plots, be cautious about making line plots using multiple semantics. While sometimes informative, they can also be difficult to parse and interpret. But even when you are only examining changes across one additional variable, it can be useful to alter both the color and style of the lines. This can make the plot more accessible when printed to black-and-white or viewed by someone with color blindness:

```py
sns.relplot(x="timepoint", y="signal", hue="event", style="event",
            kind="line", data=fmri);

```

![http://seaborn.pydata.org/_images/relational_39_0.png](img/e04dba52166d9d9333fcd7d824a3bb02.jpg)

When you are working with repeated measures data (that is, you have units that were sampled multiple times), you can also plot each sampling unit separately without distinguishing them through semantics. This avoids cluttering the legend:

```py
sns.relplot(x="timepoint", y="signal", hue="region",
            units="subject", estimator=None,
            kind="line", data=fmri.query("event == 'stim'"));

```

![http://seaborn.pydata.org/_images/relational_41_0.png](img/21df9f9baaa60a010973e5b26a1e7259.jpg)

The default colormap and handling of the legend in [`lineplot()`](../generated/seaborn.lineplot.html#seaborn.lineplot "seaborn.lineplot") also depends on whether the hue semantic is categorical or numeric:

```py
dots = sns.load_dataset("dots").query("align == 'dots'")
sns.relplot(x="time", y="firing_rate",
            hue="coherence", style="choice",
            kind="line", data=dots);

```

![http://seaborn.pydata.org/_images/relational_43_0.png](img/1e1bc5f34e003c074af116c6f2a0cd58.jpg)

It may happen that, even though the `hue` variable is numeric, it is poorly represented by a linear color scale. That’s the case here, where the levels of the `hue` variable are logarithmically scaled. You can provide specific color values for each line by passing a list or dictionary:

```py
palette = sns.cubehelix_palette(light=.8, n_colors=6)
sns.relplot(x="time", y="firing_rate",
            hue="coherence", style="choice",
            palette=palette,
            kind="line", data=dots);

```

![http://seaborn.pydata.org/_images/relational_45_0.png](img/a1749fc5a5cf2bb2e4b3c4c741c1b8f7.jpg)

Or you can alter how the colormap is normalized:

```py
from matplotlib.colors import LogNorm
palette = sns.cubehelix_palette(light=.7, n_colors=6)
sns.relplot(x="time", y="firing_rate",
            hue="coherence", style="choice",
            hue_norm=LogNorm(),
            kind="line", data=dots);

```

![http://seaborn.pydata.org/_images/relational_47_0.png](img/fe7b4873c67199b180714300c4b65def.jpg)

The third semantic, size, changes the width of the lines:

```py
sns.relplot(x="time", y="firing_rate",
            size="coherence", style="choice",
            kind="line", data=dots);

```

![http://seaborn.pydata.org/_images/relational_49_0.png](img/3ab581b389567a79a8bf4ddd79ec9356.jpg)

While the `size` variable will typically be numeric, it’s also possible to map a categorical variable with the width of the lines. Be cautious when doing so, because it will be difficult to distinguish much more than “thick” vs “thin” lines. However, dashes can be hard to perceive when lines have high-frequency variability, so using different widths may be more effective in that case:

```py
sns.relplot(x="time", y="firing_rate",
           hue="coherence", size="choice",
           palette=palette,
           kind="line", data=dots);

```

![http://seaborn.pydata.org/_images/relational_51_0.png](img/de02eb5b7f92879b7db8e43542d3bef9.jpg)

### Plotting with date data

Line plots are often used to visualize data associated with real dates and times. These functions pass the data down in their original format to the underlying matplotlib functions, and so they can take advantage of matplotlib’s ability to format dates in tick labels. But all of that formatting will have to take place at the matplotlib layer, and you should refer to the matplotlib documentation to see how it works:

```py
df = pd.DataFrame(dict(time=pd.date_range("2017-1-1", periods=500),
                       value=np.random.randn(500).cumsum()))
g = sns.relplot(x="time", y="value", kind="line", data=df)
g.fig.autofmt_xdate()

```

![http://seaborn.pydata.org/_images/relational_53_0.png](img/ec42ca3d421c2355f3f6ef3ce2b26cfe.jpg)

## Showing multiple relationships with facets

We’ve emphasized in this tutorial that, while these functions _can_ show several semantic variables at once, it’s not always effective to do so. But what about when you do want to understand how a relationship between two variables depends on more than one other variable?

The best approach may be to make more than one plot. Because [`relplot()`](../generated/seaborn.relplot.html#seaborn.relplot "seaborn.relplot") is based on the [`FacetGrid`](../generated/seaborn.FacetGrid.html#seaborn.FacetGrid "seaborn.FacetGrid"), this is easy to do. To show the influence of an additional variable, instead of assigning it to one of the semantic roles in the plot, use it to “facet” the visualization. This means that you make multiple axes and plot subsets of the data on each of them:

```py
sns.relplot(x="total_bill", y="tip", hue="smoker",
            col="time", data=tips);

```

![http://seaborn.pydata.org/_images/relational_55_0.png](img/68f664bef239fbf14e3fa497f5ff9a02.jpg)

You can also show the influence two variables this way: one by faceting on the columns and one by faceting on the rows. As you start adding more variables to the grid, you may want to decrease the figure size. Remember that the size [`FacetGrid`](../generated/seaborn.FacetGrid.html#seaborn.FacetGrid "seaborn.FacetGrid") is parameterized by the height and aspect ratio of _each facet_:

```py
sns.relplot(x="timepoint", y="signal", hue="subject",
            col="region", row="event", height=3,
            kind="line", estimator=None, data=fmri);

```

![http://seaborn.pydata.org/_images/relational_57_0.png](img/6f53de8393d846c1c350ac9a5e85c900.jpg)

When you want to examine effects across many levels of a variable, it can be a good idea to facet that variable on the columns and then “wrap” the facets into the rows:

```py
sns.relplot(x="timepoint", y="signal", hue="event", style="event",
            col="subject", col_wrap=5,
            height=3, aspect=.75, linewidth=2.5,
            kind="line", data=fmri.query("region == 'frontal'"));

```

![http://seaborn.pydata.org/_images/relational_59_0.png](img/f6baefd0e5308da156602144fc329c25.jpg)

These visualizations, which are often called “lattice” plots or “small-multiples”, are very effective because they present the data in a format that makes it easy for the eye to detect both overall patterns and deviations from those patterns. While you should make use of the flexiblity afforded by [`scatterplot()`](../generated/seaborn.scatterplot.html#seaborn.scatterplot "seaborn.scatterplot") and [`relplot()`](../generated/seaborn.relplot.html#seaborn.relplot "seaborn.relplot"), always try to keep in mind that several simple plots are usually more effective than one complex plot.